/**
 * APBUART driver for RTEMS.
 * Uses the Termios framework in interrupt-driven mode.
 */

#ifndef _DRV_LEONUART_H
#define _DRV_LEONUART_H

#include <rtems.h>
#include <rtems/libio.h>
#include <ambapp.h>

/* Statistics */
struct leonuart_stats {
    unsigned int rx, tx;
    unsigned int frame, overrun, parity, brk;
};

#define LEONUART_IOC_SET_BAUDRATE   _IOW('u', 1, unsigned int)
#define LEONUART_IOC_GET_STATS      _IOR('u', 2, struct leonuart_stats)
#define LEONUART_IOC_CLR_STATS      _IO( 'u', 3)


#define LEONUART_DRIVER_TABLE_ENTRY { leonuart_initialize, leonuart_open, leonuart_close, leonuart_read, leonuart_write, leonuart_control }

rtems_device_driver leonuart_initialize(rtems_device_major_number major,
                                    rtems_device_minor_number minor,
                                    void *arg);
rtems_device_driver leonuart_open(  rtems_device_major_number major,
                                    rtems_device_minor_number minor,
                                    void *arg);
rtems_device_driver leonuart_close( rtems_device_major_number major,
                                    rtems_device_minor_number minor,
                                    void *arg);
rtems_device_driver leonuart_read(  rtems_device_major_number major,
                                    rtems_device_minor_number minor,
                                    void *arg);
rtems_device_driver leonuart_write( rtems_device_major_number major,
                                    rtems_device_minor_number minor,
                                    void *arg);
rtems_device_driver leonuart_control(rtems_device_major_number major,
                                    rtems_device_minor_number minor,
                                    void *arg);

/**
 * Register leonuart driver.
 * Either call this function from your application at initialization time,
 * or add LEONAURT_DRIVER_TABLE_ENTRY to your device driver table.
 */
int leonuart_register(void);

/**
 * Prepare to register the specified UART devices in addition to the standard
 * devices found on the AMBA bus. This function must be called before the
 * device driver is initialized (i.e. before leonuart_register()).
 */
void leonuart_set_extra_uarts(const amba_apb_device *uarts, unsigned int n);

#endif
